using System;
using System.IO;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI;
using PluginCore.Managers;
using PluginCore.Localization;
using PluginCore;
using System.Text.RegularExpressions;
using FDTracer.Win32;
using WeifenLuo.WinFormsUI.Docking;

namespace FDTracer
{
	public class PluginUI : UserControl
    {

        private const int REFRESH_INTERVAL = 100;
        private const string PAUSE_IMAGE_ID     = "146";
        private const string RESUME_IMAGE_ID    = "147";

        private ToolStrip toolStrip;
        private ToolStripButton pauseButton;
        private ToolStripSeparator toolStripSeparator;
        private ToolStripLabel filterLabel;
        private ToolStripTextBox filterTextBox;
        private RichTextBox traceTextBox;
        
		private PluginMain pluginMain;
        private Boolean paused;
        private ImageList imageList;
        private Timer refreshTimer;
        private DateTime traceFileWriteTime;
        
		public PluginUI(PluginMain pluginMain)
		{

            this.pluginMain = pluginMain;
			this.InitializeComponent();
            this.init();
		}

        /// <summary>
        /// Accessor to the RichTextBox
        /// </summary>
        public RichTextBox Output
        {
            get { return this.traceTextBox; }
        }


        private void init()
        {
            InitializeContextMenu();

            paused = false;

            // pause / resume button icons
            imageList = new ImageList();
            imageList.ColorDepth = ColorDepth.Depth32Bit;
            imageList.TransparentColor = Color.Transparent;
            imageList.Images.Add(PluginBase.MainForm.FindImage(PAUSE_IMAGE_ID));
            imageList.Images.Add(PluginBase.MainForm.FindImage(RESUME_IMAGE_ID));
            pauseButton.Image = imageList.Images[0];
            
            // refresh timer
            refreshTimer = new Timer();
            refreshTimer.Interval = REFRESH_INTERVAL;
            refreshTimer.Tick += new EventHandler(refreshTimer_Tick);
            refreshTimer.Start();

            pauseButton.Click += new EventHandler(pauseButton_Click);
            filterTextBox.TextChanged += new EventHandler(filterTextBox_TextChanged);
        }

        // <summary>
        /// Diplays the output
        /// </summary> 
        public void DisplayOutput()
        {
            if (pluginMain.PluginSettings.AlwaysOpenPanel) this.pluginMain.PluginPanel.Show();
        }

        public void RefreshTrace(Boolean forceScroll)
        {
            Regex r;
            string logContents = getLogFileContents();
            
            char[] splitter = { '\n' };
            string[] logLines = logContents.Split(splitter);

            StringBuilder rtf = new StringBuilder("{\\rtf\\ansi{\\colortbl;\\red0\\green0\\blue0;\\red255\\green140\\blue0;\\red255\\green0\\blue0;}");
            foreach (string line in logLines)
            {
                string colorTbl = "\\cf0 ";

                if (pluginMain.PluginSettings.ColourWarnings)
                {
                    // Warning
                    r = new Regex("Warning: ", RegexOptions.IgnoreCase);
                    if (r.IsMatch(line)) colorTbl = "\\cf2 ";

                    // Error
                    r = new Regex("Error #", RegexOptions.IgnoreCase);
                    if (r.IsMatch(line)) colorTbl = "\\cf3 ";
                }
                // escape special rtf characters
                string formattedLine = line;
                formattedLine = formattedLine.Replace("\\", "\\\\");
                formattedLine = formattedLine.Replace("{", "\\{");
                formattedLine = formattedLine.Replace("}", "\\}");
                
                rtf.Append(colorTbl + formattedLine + "\\par");
            }
            rtf.Append("}");
            traceTextBox.Rtf = rtf.ToString();

            if (forceScroll)
            {
                traceTextBox.AppendText("\n"); // brings caret to the end of the textfield
                Scrolling.ScrollToBottom(traceTextBox);
            }
        }

        private void InitializeContextMenu()
        {
            ContextMenuStrip menu = new ContextMenuStrip();
            menu.Items.Add(new ToolStripMenuItem("Clear", null, new EventHandler(ClearTraceText)));
            menu.Items.Add(new ToolStripMenuItem("Copy", null, new EventHandler(CopyTraceText)));
            menu.Items.Add(new ToolStripSeparator());
            menu.Items.Add(new ToolStripMenuItem("Wrap Text", null, new EventHandler(WrapText)));
            menu.Items.Add(new ToolStripSeparator());
            menu.Items.Add(new ToolStripMenuItem("Settings...", null, new EventHandler(ShowSettings)));
            this.traceTextBox.ContextMenuStrip = menu;
        }

        void pauseButton_Click(object sender, EventArgs e)
        {
            paused = !paused;
            refreshTimer.Enabled = !paused;

            ToolStripButton button = sender as ToolStripButton;
            button.Image = imageList.Images[(paused ? 1 : 0)];
            button.ToolTipText = (paused ? "Resume Tracing" : "Pause Tracing");

            // toggle button visible forces tooltip update
            button.Visible = false;
            button.Visible = true;
        }

        void refreshTimer_Tick(object sender, EventArgs e)
        {
            DateTime writeTime = DateTime.Now;
            try 
            {
                writeTime = File.GetLastWriteTimeUtc(pluginMain.PluginSettings.TraceOutputFileName); 
            }
            catch (Exception) 
            { 
                traceFileWriteTime = writeTime;
            }

            if (traceFileWriteTime != writeTime)
            {
                traceFileWriteTime = writeTime;
                RefreshTrace(true);
            }
        }

        void filterTextBox_TextChanged(object sender, EventArgs e)
        {
            RefreshTrace(false);
        }


        private string getLogFileContents()
        {
            String contents = "";
            StreamReader sr;

            StringBuilder contentsBuilder = new StringBuilder();

            try 
            {
                sr = File.OpenText(pluginMain.PluginSettings.TraceOutputFileName);
            }
            catch (Exception)
            {
                return contents;
            }

             string s = sr.ReadLine();
             while (s != null)
             {
                if (passesFilter(s)) contentsBuilder.AppendLine(s);
                s = sr.ReadLine();
             }
            sr.Close();

            contents = contentsBuilder.ToString();

            return contents.TrimEnd(null); 
        }

        private Boolean passesFilter(string logLine)
        {
            Regex filter;
            try
            {
                filter = new Regex(filterTextBox.Text, RegexOptions.IgnoreCase);
            }
            catch (Exception)
            {
                filter = new Regex("");
            }
            return filter.IsMatch(logLine);
        }


        private void ClearTraceText(Object sender, System.EventArgs e) 
        {
            traceTextBox.Clear();

            if (File.Exists(pluginMain.PluginSettings.TraceOutputFileName))
            {
                try
                {
                    StreamWriter sw = new StreamWriter(pluginMain.PluginSettings.TraceOutputFileName);
                    sw.Write("");
                    sw.Close();
                }
                catch (Exception) {/**/}
            }

        }

        private void CopyTraceText(Object sender, System.EventArgs e) 
        {
            traceTextBox.Copy();
        }

        private void WrapText(Object sender, System.EventArgs e) 
        {
            ToolStripMenuItem item = sender as ToolStripMenuItem;
            item.Checked = !item.Checked;
            traceTextBox.WordWrap = item.Checked;
        }

        private void ShowSettings(Object sender, System.EventArgs e)
        {
            try
            {
                PluginBase.MainForm.ShowSettingsDialog(this.pluginMain.Settings, "FDTracer Settings");
            }
            catch (Exception ex)
            {
                ErrorManager.ShowError(ex);
            }
        }


		#region Windows Forms Designer Generated Code

		/// <summary>
		/// This method is required for Windows Forms designer support.
		/// Do not change the method contents inside the source code editor. The Forms designer might
		/// not be able to load this method if it was changed manually.
		/// </summary>
		private void InitializeComponent() 
        {
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(PluginUI));
            this.toolStrip = new System.Windows.Forms.ToolStrip();
            this.pauseButton = new System.Windows.Forms.ToolStripButton();
            this.toolStripSeparator = new System.Windows.Forms.ToolStripSeparator();
            this.filterLabel = new System.Windows.Forms.ToolStripLabel();
            this.filterTextBox = new System.Windows.Forms.ToolStripTextBox();
            this.traceTextBox = new System.Windows.Forms.RichTextBox();
            this.toolStrip.SuspendLayout();
            this.SuspendLayout();
            // 
            // toolStrip
            // 
            this.toolStrip.CanOverflow = false;
            this.toolStrip.GripStyle = System.Windows.Forms.ToolStripGripStyle.Hidden;
            this.toolStrip.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.pauseButton,
            this.toolStripSeparator,
            this.filterLabel,
            this.filterTextBox});
            this.toolStrip.LayoutStyle = System.Windows.Forms.ToolStripLayoutStyle.HorizontalStackWithOverflow;
            this.toolStrip.Location = new System.Drawing.Point(0, 0);
            this.toolStrip.Name = "toolStrip";
            this.toolStrip.Padding = new System.Windows.Forms.Padding(2, 0, 0, 0);
            this.toolStrip.Size = new System.Drawing.Size(280, 25);
            this.toolStrip.TabIndex = 1;
            this.toolStrip.Text = "toolStrip1";
            // 
            // pauseButton
            // 
            this.pauseButton.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.pauseButton.Image = ((System.Drawing.Image)(resources.GetObject("pauseButton.Image")));
            this.pauseButton.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.pauseButton.Margin = new System.Windows.Forms.Padding(1);
            this.pauseButton.Name = "pauseButton";
            this.pauseButton.Size = new System.Drawing.Size(23, 23);
            this.pauseButton.Text = "Pause Tracing";
            // 
            // toolStripSeparator
            // 
            this.toolStripSeparator.Name = "toolStripSeparator";
            this.toolStripSeparator.Size = new System.Drawing.Size(6, 25);
            // 
            // filterLabel
            // 
            this.filterLabel.Margin = new System.Windows.Forms.Padding(1, 1, 0, 2);
            this.filterLabel.Name = "filterLabel";
            this.filterLabel.Size = new System.Drawing.Size(35, 22);
            this.filterLabel.Text = "Filter:";
            // 
            // filterTextBox
            // 
            this.filterTextBox.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
            this.filterTextBox.Margin = new System.Windows.Forms.Padding(0, 2, 5, 2);
            this.filterTextBox.Name = "filterTextBox";
            this.filterTextBox.Overflow = System.Windows.Forms.ToolStripItemOverflow.Never;
            this.filterTextBox.Size = new System.Drawing.Size(120, 21);
            // 
            // traceTextBox
            // 
            this.traceTextBox.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.traceTextBox.BackColor = System.Drawing.Color.White;
            this.traceTextBox.BorderStyle = System.Windows.Forms.BorderStyle.None;
            this.traceTextBox.DetectUrls = false;
            this.traceTextBox.Font = new System.Drawing.Font("Courier New", 8.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.traceTextBox.Location = new System.Drawing.Point(0, 28);
            this.traceTextBox.Name = "traceTextBox";
            this.traceTextBox.ReadOnly = true;
            this.traceTextBox.Size = new System.Drawing.Size(277, 321);
            this.traceTextBox.TabIndex = 0;
            this.traceTextBox.Text = "";
            this.traceTextBox.WordWrap = false;
            // 
            // PluginUI
            // 
            this.Controls.Add(this.toolStrip);
            this.Controls.Add(this.traceTextBox);
            this.Name = "PluginUI";
            this.Size = new System.Drawing.Size(280, 352);
            this.toolStrip.ResumeLayout(false);
            this.toolStrip.PerformLayout();
            this.ResumeLayout(false);
            this.PerformLayout();

		}

		#endregion

 	}

}
